module emldata

 use param 
 use dal 
 use edetect_io_ml

 implicit none     

!   parameter            description
!   MAX_IMA              max number of images
!   MAX_INS              max number of different instruments 
!   MAX_IMSIZ            max image size 
!   MAX_BGSIZ            max background image size
!   MAX_EXPSIZ           max exposure image size
!   MAX_EVENTS           max number of events per source  
!   MAX_BANDS            max number of energy bands
!   MAX_MASK             max size of mask
!   MAX_PAR              max number of fit params
!   MAX_OFFAX            max dim of offax/pos_ang histogram
!   MAX_POSANG           dito
!   MAX_SOU              max number of sources
!   MAX_FIT              max number of sources to be fit simultaneously 


! max_ima, max_ins, max_bands are also in edetect_io !
!      integer, parameter, private :: max_ima = 54
!      integer, parameter, private :: max_ins = 9
      integer, parameter :: max_events = 200000 ! needs to be larger
!     integer, parameter, private :: max_bands = 6
!      integer, parameter :: max_fit = 10        
!      integer, parameter :: max_par = max_fit*max_ima+max_fit*3
      integer, parameter :: max_mask = 256
      integer, parameter :: max_hist = 36
      integer, parameter :: max_offax = max_hist
      integer, parameter :: max_posang = max_hist
      integer, parameter :: max_sou = 50000

!    NSOUPAR             number of global source parameters (x,y,extent)

      integer :: nspar

!   PSTEP                step width (pixels) for calculation of SCUT, ECUT

      real(kind=double), parameter :: pi=3.14159265358979323846d0, &
                                      twopi=2d0*pi,pi2=pi/2d0
      real, parameter :: pstep=0.1

! Type declarations

      type :: mask_type
        integer, dimension(max_ima,max_mask, max_mask) :: mask
        integer :: ix_min, ix_max, iy_min, iy_max
      end type mask_type

      type :: hist_type
        real, dimension(max_hist, max_hist) :: hist
        integer :: ns_offax, ne_offax, ns_posang, ne_posang
      end type hist_type

      type :: event_type
        real :: xs, ys, pixsiz
        integer :: n
        integer(int16) :: ins, ipt, offax, posang
      end type event_type 
 
!   variable             description
!   NUM_INSTR            number of instruments
!   NUM_IMAGE(MAX_INS)   number of images per instrumet
!   NIM                  total number of images
!   NUM_EVLIST(MAX_INS)  number of event lists
!   AMPL_LOW(MAX_IMA)    lower channel boundary of images  
!   AMPL_HIGH(MAX_IMA)   upper channel boundary of images
!   IP(MAX_INS,MAX_BANDS)pointer field to manage images
!   BG_SCALE             scale factor of background images
!   EX_SCALE             scale factor of exposure images  
!   MAX_SFIT             max number of sources to fit simultaneously

!*      integer, dimension(max_ins) :: num_image       
      integer, dimension(max_ins) :: num_evlist       
      integer, dimension(max_ima) :: ampl_low, ampl_high
!*      integer, dimension(max_ins, max_bands) :: ip
!*      integer :: num_instr,nim
      integer :: bg_scale,ex_scale,max_sfit,nsmul
      integer, dimension(8,max_ins) :: hrdef
      integer, dimension(max_bands,max_ins) :: xiddef 

!   IMAGE                input images
!   BG_IMAGE             background images
!   AMP_ARR
!   OFFAX_ARR
!   POSANG_ARR
!   INS_ARR
!   IPT_ARR
!   N_ARR                counts per pixel array
!   NPTS

!*      integer, dimension(:,:,:), allocatable  :: image            
!     integer, dimension(1,640,640) :: image            
      integer :: npts 

!   ECF(MAX_IMA)         energy conversion factors
!   ENERGY(MAX_IMA)      mean energies of images
!   EXPTIM(MAX_INS)      exposure times
!   XCEN(MAX_INS)        position of optical axis (x coord)
!   YCEN(MAX_INS)        position of optical axis (y coord)
!   XDCEN(MAX_INS)       pos of opt axis (x det coord)
!   YDCEN(MAX_INS)       pos of opt axis (y det coord)
!   DETPIX(MAX_INS)      det pixel size (arcsec)
!   PIXIM                image pixel size (arcsec)
!   EPSMIN(MAX_INS)      minimum off-axis angle (arcmin) 
!   EPSMAX(MAX_INS)      maximum off-axis angle (arcmin)     
!   DMLMIN               minimum maximum likelihood
!   DMLSOU               required ML improvement for multi-source fit
!   DMLEXT               required ML improvement for source extent
!   ENCES                enclosed energy for simul source fit
!   ENCEE                enclosed energy for data
!   EXP_CENTER(MAX_IMA)  on axis exposure
!   INSTRUME(MAX_IMA)    instrument name for each image
!   BAND(MAX_IMA)        number of energy band for each image

      character(len=19), dimension(max_ima) :: instrume
      integer, dimension(max_ima) :: band
!      real, dimension(max_ima) :: ecf, energy, exp_center 
      real, dimension(max_ima) :: energy, exp_center
      real, dimension(max_ins) :: exptim, xcen, ycen, xdcen, ydcen, &
                                  detpix, epsmin, epsmax, xidecf    
      integer, dimension(max_ins) :: xid_arr
      real :: pixim, dmlmin, dmlsou, dmlext, ences, encee, maxext, minext
      integer, dimension(max_fit,max_ima) :: maskzero
      real, dimension(max_fit,max_ima) :: maskfrac,psffrac
      real ::  maskthreshold=0.25,psfthreshold=0.15
! use PSF ACUURACY_EXTENDED?
      logical :: extpsf = .false.
! use PSF ACUURACY_ELLBETA? (JR)
      logical :: modpsf = .false.
! perform fit or only write output files:
      logical :: fitflag = .true.
      logical :: randomflag = .false.
      logical :: simulate = .false.
      logical :: testoutput =.false.

! factor needed to normalize the EXTENDED accuracy PSF to 1.0   
      real ::  psfnorm=795.549 
    
!   EXP_IMAGE            exposure images

      real, dimension (:,:,:), allocatable :: bg_image0
!     real, dimension (1,640,640) :: exp_image, bg_image

!  START_ARR            START and
!  STEP_ARR             STEP descriptors of input images

      REAL(kind=double), DIMENSION(2,MAX_INS) ::   START_ARR, STEP_ARR

!  NPIX_ARR             size of input images
!  RA_PNT, DEC_PNT, PA_PNT pointing direction
!  CR*                  
!  POINT_RESP(MAX_INS)  point response routines to be used 

      integer, dimension(2, max_ins) :: npix_arr
!      real(kind=double) :: ra_pnt, dec_pnt, pa_pnt
!      real(kind=double) :: crpix1,crpix2,crval1,crval2, &
!                            crdelt1,crdelt2,crota2
      integer, dimension(max_ins) :: point_resp

!  C_IMAGE(MAX_IMA)     names of input images
!  C_EVLIST(MAX_INS)    names of input photon lists
!  C_BGIMA(MAX_IMA)     names of background images
!  C_EXPIMA(MAX_IMA)    names of exposure images
!  C_D1LIST             name of input detect table
!  OUTFIL               maximum likelihood counts image
!  OUTTAB               maximum likelihood fit result table
!  EXPFLA               exposure flags 
!  PHOFLA
!  POSFLA               fit source position flag
!  EXTFLA               fit source extent flag
!  CNTFLA               fit source counts flag 
!  ERRFLA               error flag
!  XIDFLA               XID band flag

      character(1024), dimension(max_ima) :: c_image,c_bgima,c_expima, &
                                             c_detima, c_srcima
      character(1024), dimension(max_ins) :: c_evlist  
      character(1024) :: outtab, outfil, c_d1list, extmod
!-- New Parameter psfmodel - (JR) ---
      character(1024) :: psfmod
!------------------------------------
      logical :: expfla, detfla, posfla, phofla, extfla, cntfla, errfla, &
                 xidfla, hotfla, smpfla, usecalpsf, negfla, newcal, xidfix, &
                 input_ext
      integer, dimension(max_ins) :: nxidbands
      real, dimension(:,:), allocatable, public :: fima

      logical :: twostage
      logical :: threshfla 
      character(4) :: threshcol
      real :: threshold   


!   MASK
!   HIST

      type (mask_type) :: mask
      type (hist_type), dimension(max_ins) :: hist 

!   EVENT

      type (event_type), dimension(max_events) :: event

! TEST output

     integer :: count_model,count_psf
     integer :: clock1,clock2,clock3,clock0 

end module emldata
